"""
This module contains functionality for defining antimalarial drug interventions.
"""

from emod_api import schema_to_class as s2c
from emodpy_malaria.interventions.common import add_campaign_event


def add_antimalarial_drug(
        campaign,
        start_day: int = 1,
        node_ids: list = None,
        demographic_coverage: float = 1.0,
        drug_type: str = "Chloroquine",
        cost_to_consumer: float = 0):
    """
    Add an antimalarial drug intervention to your campaign. This is equivalent to 
    :doc:`emod-malaria:parameter-campaign-individual-antimalarialdrug`.

    Args:
        campaign: campaign object to which the intervention will be added, and schema_path container
        start_day: The day of the simulation on which the drug is distributed. We recommend 
            aligning this with the start of the simulation.
        node_ids: List of nodes to which distribute the intervention. If not provided, the intervention
            is distributed to all the nodes.
        demographic_coverage: The proportion of the population that will receive the drug.
        drug_type: The name of the drug to distribute in a drug intervention. Possible values are
            contained in **Malaria_Drug_Params**
        cost_to_consumer: The cost to consumer per unit distributed (unamortized)

    Returns:
        The intervention event.
    """
    antimalarial_drug_intervention = antimalarial_drug_config(campaign, drug_type=drug_type,
                                                              cost_to_consumer=cost_to_consumer)
    add_campaign_event(campaign, start_day=start_day, demographic_coverage=demographic_coverage,
                       node_ids=node_ids, intervention=antimalarial_drug_intervention)


def antimalarial_drug_config(campaign, drug_type, cost_to_consumer=1.0):
    """
    Configures antimalarial drug intervention to your campaign. This is equivalent to
    :doc:`emod-malaria:parameter-campaign-individual-antimalarialdrug`.

    Args:
        campaign: The :py:obj:`emod_api:emod_api.campaign` object to which the intervention will be added,
            and schema_path container
        drug_type: The name of the drug to distribute in a drugs intervention.
            Possible values are contained in **Malaria_Drug_Params**.
        cost_to_consumer: The cost to consumer per unit distributed (unarmortized)

    Returns:
      The configured antimalarial drug intervention.
    """
    schema_path = campaign.schema_path
    intervention = s2c.get_class_with_defaults("AntimalarialDrug", schema_path)
    intervention.Drug_Type = drug_type
    intervention.Cost_To_Consumer = cost_to_consumer
    return intervention


def new_intervention_as_file(campaign, start_day, filename=None):
    """
    Take an :doc:`emod-malaria:parameter-campaign-individual-antimalarialdrug`
    intervention from a JSON file and add it to your campaign.

    Args:
        campaign: The :py:obj:`emod_api:emod_api.campaign` object to which the intervention will be added,
            and schema_path container
        start_day: The day of the simulation on which the drug is distributed. We recommend 
            aligning this with the start of the simulation.
        filename: The JSON file that contains the intervention.

    Returns:
        The filename.
    """
    add_antimalarial_drug(campaign=campaign, start_day=start_day)
    if filename is None:
        filename = "AntimalarialDrug.json"
    campaign.save(filename)
    return filename
